var emitter = /** @class */ (function () {
    function emitter() {
        if (!window.hasOwnProperty('__emitter_map__')) {
            Object.defineProperty(window, '__emitter_map__', { value: { '*': [] } });
        }
    }
    emitter.prototype.on = function (type, handle, once) {
        if (once === void 0) { once = false; }
        var m = window.__emitter_map__;
        var typeName = once ? type + "-once" : type;
        if (!m.hasOwnProperty(typeName)) {
            Object.defineProperty(m, typeName, { value: [], writable: true });
        }
        m[typeName].push(handle);
    };
    emitter.prototype.once = function (type, handle) {
        if (type === '*') {
            throw new Error("The '*' type on 'once' is not supported");
        }
        this.on(type, handle, true);
    };
    emitter.prototype.off = function (type, handle) {
        var then = Promise ? function (cb) { return Promise.resolve().then(cb); } : setTimeout;
        then(function () {
            var m = window.__emitter_map__;
            if (!handle) {
                if (m[type]) {
                    m[type] = [];
                }
                if (m[type + "-once"]) {
                    m[type + "-once"] = [];
                }
            }
            else {
                if (m[type]) {
                    m[type].splice(m[type].indexOf(handle) >>> 0, 1);
                }
                if (m[type + "-once"]) {
                    m[type + "-once"].splice(m[type + "-once"].indexOf(handle) >>> 0, 1);
                }
            }
        });
    };
    emitter.prototype.emit = function (type, payload) {
        var _this = this;
        var m = window.__emitter_map__;
        var offList = [];
        if (m.hasOwnProperty(type)) {
            m[type].forEach(function (handle) { return handle(payload); });
        }
        if (m.hasOwnProperty(type + "-once")) {
            m[type + "-once"].forEach(function (handle) {
                handle(payload);
                offList.push(handle);
            });
        }
        m['*'].forEach(function (handle) { return handle(type, payload); });
        offList.forEach(function (item) { return _this.off(type + "-once", item); });
    };
    return emitter;
}());
export default new emitter();
